
#define MAXSTARS	4000
#define MAXFIBERS	200
#define F_MAXFIBERS	200.0
#define MAXSLIT		110


#define OUTPUT_SUFFIX  ".coords" 


#ifndef VOIDBUTTON
#define VOIDBUTTON -1
#endif


#define NORTH		-1	/*  direction of table motion WRT sky */
#define EAST		 1

#define SYSTEM		"fk4"


#ifndef TWOPI

#define TWOPI		M_PI * 2.0 

#endif

#define S2R		0.72722052166430398E-4
#define AS2R		0.48481368110953599E-5
#define DEG2RAD		0.0174532925
#define A2RAD		( DEG2RAD / 3600.0 )
#define MIN2R		( AS2R * 60.0 )

#define FtoK( t )	( 273.15 + ( ( t - 32.0 ) * 100.0 / 180.0 ) )



/********   This is the value you change **********/

#define ENCODER_SCALE	400.0				/* steps/mm */

#define MOTOR_RESOLUTION  ( ENCODER_SCALE * 25.4 )	/*  steps/inch */


#define WIYN_SCALE	9.4000  /* Phil's value = 9.4032 */
#define WIYN_C		81.0
#define WIYN_RADIUS	5675.2


#define HYDRA_SCALE     ( ENCODER_SCALE / WIYN_SCALE / AS2R )

/* old values for historical purposes */
/*#define PLATE_SCALE	6.6140 */
/* #define MOTOR_RESOLUTION  10160.0 	  steps/inch */

#define RADIUS		3302.0
#define WAVECAM		6000.0

#define  NAME_FLD_WIDTH  20

#define REFRACT_PTS   10
#define REFRACT_WAVE0 6000.0

#define KPNO_LAT	31.9633
#define KPNO_ALT	2064.0
#define KPNO_ATM_PRES	795.0
#define KPNO_HUMIDITY	0.60
#define LAPSE_RATE	0.0065 

#define WHITE_SPOT_LAT	-20.0

#define REFRACT_PREC	1.0E-10

#define FOCUS_RADIUS  140.7	/* Radius at which focus will be set */
#define FOCUS_STEP      2.0	/* Height of step used for focus */
#define FIB_FOCUS_HT    3.5	/* "optical" height of buttons */

#define STEP1RADIUS 127.0	/* Radius of plate step in mm */
#define STEP2RADIUS 177.8

#define STEP1HEIGHT 2.0		/* Height of plate step in mm. */
#define STEP2HEIGHT 4.0

#define PUPIL  12700.0		/* pupil radius from focal plane */

/****  new wiyn stuff ****/

#define A_CORR	6.391006	/* coefficients for corrector polynomial */
#define B_CORR	1.249909E-4
#define C_CORR	2.576783E-5
#define D_CORR	4.136236E-7

#define RADIUS_FOCAL	5675.2		/* radius, pupil to f.p. */
#define RADIUS_EFF	5680.2875


/****  end new wiyn stuff ****/


#define EWSIGN  1.0		/*  in case sign of output must be flipped */
#define NSSIGN  -1.0		/*  in case sign of output must be flipped */

#ifndef TRUE
#define TRUE  (1)
#define FALSE (0)
#endif

#define NORMAL_ASTROMETRY  1
#define REVERSE_ASTROMETRY 0

#define SPACE		'\040'
#define RETURN		'\015'
#define NEWLINE		'\n'

#define UNSORTED -2
#define SORTED   -1
#define UNASSIGNED -1

#define ARGLENGTH 128

struct  skyCoord{
	 double ra;
	 double dec;
};

typedef struct skyCoord aCoord;

struct starinfo{

	char name[64];
	int internalID;
	int IDnumber;		/*  number assigned to object in file */
	char objectType;
	int button;		/*  button assigned to this object */
	double ra;		/*  "raw" ra & dec in radians */
	double dc;		
	double ran;		/*  ra & dec in current epoch  */
	double dcn;
	double ax;		/*  plate coords of object */
	double ay;
	double rar;		/*  average refraction  */
	double dcr;
	double rarsig;
	double dcrsig;
	double rardif;
	double dcrdif;
	double xm;
	double ym;
	double adxp;		/*  x position in motor steps at spec. wave*/
	double adyp;		/*  y position in motor steps at spec. wave*/

	aCoord  rawCoord;
	aCoord  precessedCoord;

	aCoord  specRefractedCoord;	/* coordinates at spectrograph wavelength*/
	point  specProjectedCoord;
	point  specFlattenedCoord;
	point  specCorrectedCoord;
	point  specRotatedCoord;
	point  specUnWarpedCoord;
	point  specEncoderCoord;


	aCoord  camRefractedCoord;	/* coordinates at Camera wavelength*/

	point  camProjectedCoord;
	point  camFlattenedCoord;
	point  camCorrectedCoord;
	point  camRotatedCoord;
	point  camUnWarpedCoord;
	point  camEncoderCoord;

	double afoc;		/*  focus error in mm */
	double r;		/*  radius from field center to object */
	double theta;		/*  angle +/- pi of object */
	double numa;
};

typedef struct starinfo aStar;

struct aComment{
   char *theText;
   struct aComment *nextComment;
};


struct exposinfo{

	char fileName[128];
	char expName[64];
	char cable[64];
	char weighting[64];

	struct aComment *nextComment;

	double epoch;
	double current;

	double exptime;
	double st;

	double month;

	double wave;
	double guidewave;

	double tempFactor;
	double tempadopt;
	double tempfudge;

	double worstra;
	double worstdc;
};


/* this is the structure passed by the tcl interpreter */

struct localData{
   struct starinfo  **stars;
   struct exposinfo *pExposure;
   FILE  *dataFile;
};

   

extern struct starinfo *Stars[];	/* main "star" structure */

extern struct exposinfo Exposure;

extern int direct;		/* mode flags for nessie */
extern int immortal;
extern int verbose;

extern struct exposinfo Exposure;	/* exposure information */

extern struct termios ttyNormal;
extern struct termios ttyNonCanon;

extern double stor();
extern double astor();

extern long secToSteps();

extern void properM();
extern void precess();
extern void aveRefract();
extern void project();
extern void flatten();
extern void correctIt();
extern void unCorrectIt();
extern void rotateIt();
extern void unWarpIt();
extern void encoderIt();

extern double Pterm();
extern double Rterm();
extern double Zrefract();
	

